
/* ------------------------------------------------------------------------- */

/*
 * Copyright (c) 1999
 *      GMRS Software GmbH, Innsbrucker Ring 159, 81669 Munich, Germany.
 *      http://www.gmrs.de
 *      All rights reserved.
 *      Author: Arno Unkrig (arno.unkrig@gmrs.de)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by GMRS Software GmbH.
 * 4. The name of GMRS Software GmbH may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY GMRS SOFTWARE GMBH ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GMRS SOFTWARE GMBH BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/* ------------------------------------------------------------------------- */

#ifndef __HTMLControl_h_INCLUDED__ /* { */
#define __HTMLControl_h_INCLUDED__

/* ------------------------------------------------------------------------- */

#ident "$Id: HTMLControl.h,v 1.6 1999/09/21 09:03:14 arno Exp $"

#include "HTMLParser.h"

class istream;

/* ------------------------------------------------------------------------- */

class HTMLControl : public HTMLParser {

public:
  HTMLControl(istream &is_, bool debug_scanner_, bool debug_parser_) :
    HTMLParser(),
    current_line(1),
    current_column(0),
    literal_mode(false),
    next_token(EOF),
    debug_scanner(debug_scanner_),
    is(is_),
    number_of_ungotten_chars(0)
  { yydebug = debug_parser_; }

  int current_line;
  int current_column;

private:

  /*
   * Implementing virtual methods of "HTMLParser".
   */
  /*virtual*/ int yylex(yy_HTMLParser_stype *value_return);
  /*virtual*/ bool read_cdata(const char *terminal, string *value_return);

  /*
   * Helpers.
   */
  int yylex2(yy_HTMLParser_stype *value_return, int *tag_type_return);
  bool                literal_mode;
  int                 next_token;
  yy_HTMLParser_stype next_token_value;
  int                 next_token_tag_type;

  int get_char();
  void unget_char(int);

  bool debug_scanner;

  istream &is;
  int     ungotten_chars[5];
  int     number_of_ungotten_chars;
};

/* ------------------------------------------------------------------------- */

#endif /* } */

/* ------------------------------------------------------------------------- */

